%% Comparison between gradients: finite differences and adjoint variable method
% This code contains the comparison between the gradient of Eq.(39) in the    
% paper "Adjoint sensitivity analysis and optimization of hysteretic
% dynamic systems with nonlinear viscous dampers" by Nicolo Pollini, Oren
% Lavan, and Oded Amir, calculated with the finite differences method and 
% the adjoint variable method.
% 
% Technion - Israel Institute of Technology, Haifa, Israel, 2017
%
% Contacts: Nicolo Pollini nicolo@campus.technion.ac.il
%           Oren Lavan lavan@technion.ac.il
%           Oded Amir odedamir@technion.ac.il



clear all
close all
clc
commandwindow;
global cd kd coeff_alpha ratio Ns ksl T Fy


fprintf('Comparison between gradients: finite differences and adjoint variable method \n')
fprintf('Gradient of Eq.(39) in the paper "Adjoint sensitivity analysis and \n')
fprintf('optimization of hysteretic dynamic systems with nonlinear viscous dampers" \n')
fprintf('by Nicolo Pollini, Oren Lavan, and Oded Amir \n')
fprintf(' \n')
fprintf('Technion - Israel Institute of Technology, Haifa, Israel, 2017 \n')
fprintf(' \n')
fprintf('Contacts: Nicolo Pollini nicolo@campus.technion.ac.il \n')
fprintf('          Oren Lavan lavan@technion.ac.il \n')
fprintf('          Oded Amir odedamir@technion.ac.il \n')
fprintf(' \n')
%% Input data

load M
load K
load T
load H
load Fy

%--------------------------------------------------------------------------
% Frame data

% Mass matrix
m=T'*M*T/1000; % [10^3 ton]
% Stiffness matrix
ks=T'*K*T/1000; % [kN/mm]
% Local stiffness matrix
ksl=K/1000; % [kN/mm]
% Number of degrees of freedom
Ndof=size(ks,1);
% Story hight
Lh = 3000; % [mm]
% Story width
Lb = 5000; % [mm]
% Exponent for stiffness model of Sivaselvan and Reinhorn
% Reference: Sivaselvan, M. V., & Reinhorn, A. M. (2000). Hysteretic models
% for deteriorating inelastic structures. Journal of Engineering Mechanics,
% 126(6), 633-640.
Ns=50;
% Eignervalues of the system
A=ks\m;
[modes,eigenval]=eig(A);
omega_2=1./diag(eigenval);
[omega2,indice]=sort(omega_2);
% Circular frequency
omega=omega2.^0.5;
% Structural damping ratio (typically 2-5%)
ratio_s=0.05;
%Rayleigh damping matrix of the structure
a0=ratio_s*2*omega(1)*omega(2)/(omega(1)+omega(2));
a1=ratio_s*2/(omega(1)+omega(2));
cs=a0*m+a1*ks;
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Dampers data

% Number of dampers
Ndamper=size(H,1);
% Angle of the damper diagonal element with respect to a vertical axis 
gammad = asind(Lb/(Lh^2+Lb^2)^0.5);
% Exponent for the fractional power law for the relation between damper
% force and velocity
coeff_alpha = 0.35;
% Damping coefficient of the damper
coeff_c = 100; %kN(s/mm)^alpha
% Ratio between stiffness of the damper and the brace, and the damping
% coefficient of the damper. 
% For more details see: Pollini, N., Lavan, O., & Amir, O. (2017). Minimum?
% cost optimization of nonlinear fluid viscous dampers and their supporting
% members for seismic retrofitting. Earthquake Engineering & Structural 
% Dynamics.
ratio=1.1042;
% Stiffness coefficient accounting for the stiffness of the supporting
% brace and of the damper
coeff_k =coeff_c*ratio ; % [kN/mm]

alpha = coeff_alpha*ones(Ndamper,1); % Column vector, in case one allows different exponents for the dampers
cd = diag(coeff_c*ones(Ndamper,1)); % [kN(s/mm)^alpha] - Diagonal matrix. The two dampers here have the same dampoing coefficient
kd = diag(coeff_k*ones(Ndamper,1)); % [kN/mm] - Diagonal matrix. The two dampers elements here have the same stiffness coefficient
%--------------------------------------------------------------------------

% Initia values at t=0
initial_position = 0*ones(Ndof,1); % [mm] Displacements
initial_velocity = 0*ones(Ndof,1); % [mm/s] Velocities
initial_Fd=0*ones(Ndamper,1); % [kN] Forces in the dampers

% Allowable inter-story drift (i.e. relative displacement between contiguous floors)
dis_all=9; % [mm]

%% Gradient calculated with the finite difference method

% Finite difference step-size
h=1e-5;
% Parameter for max approximation of Eq.(37), Eq.(43), Eq.(44)
r=1000;
% Parameter for max approximation of Eq.(39)
q=1000;

% Structural response for cd = [cd1 cd2], and kd = rho*[cd1 cd2]
[dis1,vel,time1,Fd,Fcol]=run(m,cs,ks,ksl,cd,kd,alpha,gammad,initial_position,initial_velocity,initial_Fd);
save analysis dis1 vel time1 Fd Fcol T H % We save the analysis output, we will use it in the adjoint analysis
[d1]=dcpnorm(time1,dis_all,dis1,r,q,H);

% Structural response for cd = [cd1+h cd2], and kd = rho*[cd1+h cd2]
[dis2,~,time2,~,~]=run(m,cs,ks,ksl,cd+h*[1 0; 0 0],kd+h*[ratio 0; 0 0],alpha,gammad,initial_position,initial_velocity,initial_Fd);
[d2]=dcpnorm(time2,dis_all,dis2,r,q,H);

% Structural response for cd = [cd1 cd2+h], and kd = rho*[cd1 cd2+h]
[dis3,~,time3,~,~]=run(m,cs,ks,ksl,cd+h*[0 0; 0 1],kd+h*[0 0; 0 ratio],alpha,gammad,initial_position,initial_velocity,initial_Fd);
[d3]=dcpnorm(time3,dis_all,dis3,r,q,H);

GradFiniteDiff=zeros(1,2);
GradFiniteDiff(1)=(d2-d1)/h;
GradFiniteDiff(2)=(d3-d1)/h;
fprintf('The gradient calculated with the finite differences is: %s\n',num2str(GradFiniteDiff))

%% Gradient calculated with the adjoint variable method "discretize-then-differentiate"

Nh=size(Fcol,1);                 
[grad]=DiscrDiff(m,cs,ksl,cd,kd,coeff_alpha,Ndamper,Ndof,Nh,gammad,dis_all,r,q);

fprintf('The gradient calculated with the adjoint method is: %s\n',num2str(grad))

fprintf('The ratio between the two gradients is: %s\n',num2str(GradFiniteDiff./grad))







